@@ -488,7 +488,7 @@ TableSubgroupGLM <- function(formula, var_subgroup = NULL, var_cov = NULL, data,
           error = function(e) NA
         )
         # Calculate pv_int
-        if (is.na(model.int) || !inherits(model.int, "merMod")) { # Check if model is invalid or not an S4 object
+        if (!inherits(model.int, "merMod")) { # Check if model is invalid or not an S4 object
           pv_int <- NA
         } else {
           coef_names <- names(lme4::fixef(model.int))
@@ -880,4 +880,4 @@ TableSubgroupMultiGLM <- function(formula, var_subgroups = NULL, var_cov = NULL,
       return(rbind(out.all, out.list %>% dplyr::bind_rows()))
     }
   }
-}
\ No newline at end of file
+}
