@@ -17,18 +17,35 @@
 #' @rdname svyglm.display
 #' @export
 #' @importFrom survey svyglm
-#' @importFrom stats update confint
+#' @importFrom stats update confint terms getCall
 
 
 svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL) {
   model <- svyglm.obj
   design.model <- model$survey.design
-  xs <- attr(model$terms, "term.labels")
+  terms_obj <- stats::terms(model)
+  xs <- attr(terms_obj, "term.labels")
+  offset_idx <- attr(terms_obj, "specials")$offset
+  offset_terms <- if (!is.null(offset_idx) && length(offset_idx) > 0) {
+    xs[offset_idx]
+  } else {
+    character(0)
+  }
+  if (length(offset_terms) == 0) {
+    offset_terms <- xs[grepl("^offset\\(", xs)]
+  }
+  xs <- setdiff(xs, offset_terms)
   y <- names(model$model)[1]
   gaussianT <- ifelse(length(grep("gaussian", model$family)) == 1, T, F)
+  current_offset <- stats::getCall(model)$offset
   
   model_data <- model.frame(model)
-  categorical_vars <- xs[sapply(model_data[xs], function(x) is.factor(x) | is.character(x))]
+  base_vars <- xs[!grepl(":", xs) & xs %in% names(model_data)]
+  if (length(base_vars) > 0) {
+    categorical_vars <- base_vars[sapply(model_data[, base_vars, drop = FALSE], function(x) is.factor(x) | is.character(x))]
+  } else {
+    categorical_vars <- character(0)
+  }
   
   
 
@@ -50,9 +67,19 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
     rownames(uni.res) <- rownames(uni)
     res <- uni.res
   } else {
+    update_args <- list(
+      object = model,
+      formula = stats::formula(paste(c(". ~ .", xs), collapse = " - ")),
+      design = design.model
+    )
+    if (!is.null(current_offset)) {
+      update_args$offset <- current_offset
+    }
+    basemodel <- do.call(stats::update, update_args)
     uni <- lapply(xs, function(v) {
-      coef.df <- data.frame(coefNA(stats::update(model, formula(paste(paste(c(". ~ .", xs), collapse = " - "), " + ", v)), design = design.model)))[-1, ]
-      confint.df <- data.frame(stats::confint(stats::update(model, formula(paste(paste(c(". ~ .", xs), collapse = " - "), " + ", v)), design = design.model)))[-1, ]
+      uni_model <- stats::update(basemodel, stats::formula(paste0(". ~ . +", v)), design = design.model)
+      coef.df <- data.frame(coefNA(uni_model))[-1, ]
+      confint.df <- data.frame(stats::confint(uni_model))[-1, ]
 
       conf.df <- data.frame(matrix(nrow = nrow(coef.df), ncol = 2))
       rownames(conf.df) <- rownames(coef.df)
@@ -82,7 +109,8 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
         mul.res <- t(rbind(mul.summ, ifelse(mul[, 4] <= 0.001, "< 0.001", as.character(round(mul[, 4], decimal + 1)))))
         colnames(mul.res) <- c(paste("adj. coeff.(", 100 - 100 * 0.05, "%CI)", sep = ""), "adj. P value")
       }else{
-        significant_vars <- rownames(uni)[as.numeric(uni[, 4]) < pcut.univariate]
+        significant_coefs <- rownames(uni)[as.numeric(uni[, 4]) < pcut.univariate]
+        terms_to_include <- character(0)
         
         if (length(categorical_vars) != 0){
           factor_vars_list <- lapply(categorical_vars, function(factor_var) {
@@ -101,13 +129,42 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
             p_values <- uni[variables, 4]
             
             if (any(p_values < pcut.univariate, na.rm = TRUE)) {
-              significant_vars <- setdiff(significant_vars, variables)
-              
-              significant_vars <- unique(c(significant_vars, key))
+              terms_to_include <- unique(c(terms_to_include, key))
             }
           }
         }
         
+        for (term in xs) {
+          if (grepl(":", term)) {
+            interaction_parts <- strsplit(term, ":")[[1]]
+            interaction_coefs <- rownames(uni)[grepl(term, rownames(uni), fixed = TRUE)]
+            
+            if (length(interaction_coefs) == 0) {
+              interaction_mask <- rep(TRUE, nrow(uni))
+              for (part in interaction_parts) {
+                interaction_mask <- interaction_mask & grepl(part, rownames(uni), fixed = TRUE)
+              }
+              interaction_mask <- interaction_mask & grepl(":", rownames(uni), fixed = TRUE)
+              interaction_coefs <- rownames(uni)[interaction_mask]
+            }
+            
+            if (length(interaction_coefs) > 0) {
+              p_values <- uni[interaction_coefs, 4]
+              
+              if (any(as.numeric(p_values) < pcut.univariate, na.rm = TRUE)) {
+                main_effects <- strsplit(term, ":")[[1]]
+                terms_to_include <- unique(c(terms_to_include, main_effects, term))
+              }
+            }
+          } else if (!(term %in% categorical_vars)) {
+            if (term %in% significant_coefs) {
+              terms_to_include <- unique(c(terms_to_include, term))
+            }
+          }
+        }
+        
+        significant_vars <- xs[xs %in% terms_to_include]
+        
         if (length(significant_vars) == 0 ){
           
           mul.res <- matrix(NA, nrow = nrow(uni.res), ncol = 2)
@@ -115,8 +172,16 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
           rownames(mul.res) <- rownames(uni.res)
           
         }else{
-          selected_formula <- as.formula(paste(y, "~", paste(significant_vars, collapse = " + ")))
-          selected_model <- survey::svyglm(selected_formula, design = design.model ) 
+          selected_rhs <- significant_vars
+          if (length(offset_terms) > 0) {
+            selected_rhs <- c(selected_rhs, offset_terms)
+          }
+          selected_formula <- as.formula(paste(y, "~", paste(selected_rhs, collapse = " + ")))
+          glm_args <- list(formula = selected_formula, design = design.model, family = model$family)
+          if (!is.null(current_offset)) {
+            glm_args$offset <- current_offset
+          }
+          selected_model <- do.call(survey::svyglm, glm_args)
           mul <- cbind(coefNA(selected_model)[-1, , drop = FALSE], stats::confint(selected_model)[-1, , drop = FALSE])
           mul.summ <- paste(round(mul[, 1], decimal), " (", round(mul[, 5], decimal), ",", round(mul[, 6], decimal), ")", sep = "")
           mul.res1 <- t(rbind(mul.summ, ifelse(mul[, 4] <= 0.001, "< 0.001", as.character(round(mul[, 4], decimal + 1)))))
@@ -150,7 +215,8 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
         mul.res <- t(rbind(mul.summ, ifelse(mul[, 4] <= 0.001, "< 0.001", as.character(round(mul[, 4], decimal + 1)))))
         colnames(mul.res) <- c(paste("adj. OR.(", 100 - 100 * 0.05, "%CI)", sep = ""), "adj. P value")
       }else{
-        significant_vars <- rownames(uni)[as.numeric(uni[, 4]) < pcut.univariate]
+        significant_coefs <- rownames(uni)[as.numeric(uni[, 4]) < pcut.univariate]
+        terms_to_include <- character(0)
         
         if (length(categorical_vars) != 0){
           factor_vars_list <- lapply(categorical_vars, function(factor_var) {
@@ -169,21 +235,58 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
             p_values <- uni[variables, 4]
             
             if (any(p_values < pcut.univariate, na.rm = TRUE)) {
-              significant_vars <- setdiff(significant_vars, variables)
-              
-              significant_vars <- unique(c(significant_vars, key))
+              terms_to_include <- unique(c(terms_to_include, key))
             }
           }
         }
         
+        for (term in xs) {
+          if (grepl(":", term)) {
+            interaction_parts <- strsplit(term, ":")[[1]]
+            interaction_coefs <- rownames(uni)[grepl(term, rownames(uni), fixed = TRUE)]
+            
+            if (length(interaction_coefs) == 0) {
+              interaction_mask <- rep(TRUE, nrow(uni))
+              for (part in interaction_parts) {
+                interaction_mask <- interaction_mask & grepl(part, rownames(uni), fixed = TRUE)
+              }
+              interaction_mask <- interaction_mask & grepl(":", rownames(uni), fixed = TRUE)
+              interaction_coefs <- rownames(uni)[interaction_mask]
+            }
+            
+            if (length(interaction_coefs) > 0) {
+              p_values <- uni[interaction_coefs, 4]
+              
+              if (any(as.numeric(p_values) < pcut.univariate, na.rm = TRUE)) {
+                main_effects <- strsplit(term, ":")[[1]]
+                terms_to_include <- unique(c(terms_to_include, main_effects, term))
+              }
+            }
+          } else if (!(term %in% categorical_vars)) {
+            if (term %in% significant_coefs) {
+              terms_to_include <- unique(c(terms_to_include, term))
+            }
+          }
+        }
+        
+        significant_vars <- xs[xs %in% terms_to_include]
+        
         if (length(significant_vars) == 0 ){
           mul.res <- matrix(NA, nrow = nrow(uni.res), ncol = 2)
           colnames(mul.res) <- c(paste("adj. coeff.(", 100 - 100 * 0.05, "%CI)", sep = ""), "adj. P value")
           rownames(mul.res) <- rownames(uni.res)
           
         }else{
-          selected_formula <- as.formula(paste(y, "~", paste(significant_vars, collapse = " + ")))
-          selected_model <- survey::svyglm(selected_formula, design = design.model ) 
+          selected_rhs <- significant_vars
+          if (length(offset_terms) > 0) {
+            selected_rhs <- c(selected_rhs, offset_terms)
+          }
+          selected_formula <- as.formula(paste(y, "~", paste(selected_rhs, collapse = " + ")))
+          glm_args <- list(formula = selected_formula, design = design.model, family = model$family)
+          if (!is.null(current_offset)) {
+            glm_args$offset <- current_offset
+          }
+          selected_model <- do.call(survey::svyglm, glm_args)
           mul <- cbind(coefNA(selected_model)[-1, , drop = FALSE], stats::confint(selected_model)[-1, , drop = FALSE])
           mul.summ <- paste(round(exp(mul[, 1]), decimal), " (", round(exp(mul[, 5]), decimal), ",", round(exp(mul[, 6]), decimal), ")", sep = "")
           mul.res1 <- t(rbind(mul.summ, ifelse(mul[, 4] <= 0.001, "< 0.001", as.character(round(mul[, 4], decimal + 1)))))
@@ -236,18 +339,35 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
     rn.list[[x]] <<- paste(xs[x], ": ", model$xlevels[[xs[x]]][2], " vs ", model$xlevels[[xs[x]]][1], sep = "")
   })
   lapply(varnum.mfac, function(x) {
-    if (grepl(":", xs[x])) {
-      a <- unlist(strsplit(xs[x], ":"))[1]
-      b <- unlist(strsplit(xs[x], ":"))[2]
-
-      if (a %in% xs && b %in% xs) {
-        ref <- paste0(a, model$xlevels[[a]][1], ":", b, model$xlevels[[b]][1])
-        rn.list[[x]] <<- c(paste(xs[x], ": ref.=", ref, sep = ""), gsub(xs[x], "   ", rn.list[[x]]))
+    var_name <- xs[x]
+    if (grepl(":", var_name)) {
+      components <- unlist(strsplit(var_name, ":"))
+      are_all_factors <- all(sapply(components, function(comp) comp %in% names(model$xlevels)))
+      
+      if (are_all_factors) {
+        ref <- paste(sapply(components, function(comp) model$xlevels[[comp]][1]), collapse = ":")
+        rn.list[[x]] <<- c(paste(var_name, ": ref.=", ref, sep = ""), gsub(var_name, "   ", rn.list[[x]]))
       } else {
-        rn.list[[x]] <<- c(paste(xs[x], ": ref.=NA", sep = ""), gsub(xs[x], "   ", rn.list[[x]]))
+        factor_comp <- components[components %in% names(model$xlevels)]
+        if (length(factor_comp) > 0) {
+          multi_level_factors <- factor_comp[sapply(factor_comp, function(f) length(model$xlevels[[f]]) > 2)]
+          if (length(multi_level_factors) > 0) {
+            ref_levels <- sapply(multi_level_factors, function(f) model$xlevels[[f]][1])
+            ref_string <- paste(ref_levels, collapse = ",")
+            rn.list[[x]] <<- c(paste(var_name, ": ref.=", ref_string, sep = ""), gsub(var_name, "   ", rn.list[[x]]))
+          } else {
+            rn.list[[x]] <<- c(var_name, gsub(var_name, "   ", rn.list[[x]]))
+          }
+        } else {
+          rn.list[[x]] <<- c(var_name, gsub(var_name, "   ", rn.list[[x]]))
+        }
       }
     } else {
-      rn.list[[x]] <<- c(paste(xs[x], ": ref.=", model$xlevels[[xs[x]]][1], sep = ""), gsub(xs[x], "   ", rn.list[[x]]))
+      if (var_name %in% names(model$xlevels)) {
+        rn.list[[x]] <<- c(paste(var_name, ": ref.=", model$xlevels[[var_name]][1], sep = ""), gsub(var_name, "   ", rn.list[[x]]))
+      } else {
+        rn.list[[x]] <<- c(var_name, gsub(var_name, "   ", rn.list[[x]]))
+      }
     }
     # rn.list[[x]] <<- c(paste(xs[x], ": ref.=", model$xlevels[[xs[x]]][1], sep = ""), gsub(xs[x], "   ", rn.list[[x]]))
   })
@@ -289,4 +409,3 @@ svyregress.display <- function(svyglm.obj, decimal = 2, pcut.univariate = NULL)
   class(results) <- c("display", "list")
   return(results)
 }
-
